#ifndef _MAIN_H
#define _MAIN_H
#include <stdio.h>
#include <string.h>
//define some data type shortcuts
typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned long u32;
typedef signed char s8;
typedef signed short s16;
typedef signed long s32;
//declare some function prototypes
void DrawPixel3(int, int, unsigned short);
void DrawBox3(int, int, int, int, unsigned short)
void DrawChar(int, int, char, unsigned short);
void Print(int, int, char *, unsigned short);
//define the timer constants
#define TIMER_FREQUENCY_SYSTEM 0x0
#define TIMER_FREQUENCY_64 0x1
#define TIMER_FREQUENCY_256 0x2
#define TIMER_FREQUENCY_1024 0x3
#define TIMER_OVERFLOW 0x4

#define TIMER_ENABLE 0x80
#define TIMER_IRQ_ENABLE 0x40
//define the timer status addresses
#define REG_TM0CNT *(volatile u16*)0x4000102
#define REG_TM1CNT *(volatile u16*)0x4000106
#define REG_TM2CNT *(volatile u16*)0x400010A
#define REG_TM3CNT *(volatile u16*)0x400010E
//define the timer data addresses
#define REG_TM0D *(volatile u16*)0x4000100
#define REG_TM1D *(volatile u16*)0x4000104
#define REG_TM2D *(volatile u16*)0x4000108
#define REG_TM3D *(volatile u16*)0x400010C
//define some video mode values
#define REG_DISPCNT *(unsigned long*)0x4000000
#define MODE_3 0x3
#define BG2_ENABLE 0x400
//declare scanline counter for vertical blank
volatile unsigned short* ScanlineCounter ;

//create a pointer to the video buffer
unsigned short* videoBuffer;
//////////////////////////////////////////////////
// Function: Print
// Prints a string using the hard-coded font
//////////////////////////////////////////////////
void Print(int left, int top, char *str, unsigned short color){
	int pos = 0;

	while (*str)
	{
		DrawChar(left + pos, top, *str++, color);
		pos += 8;
}
}
//////////////////////////////////////////////////
// Function: DrawChar
// Draws a character one pixel at a time
//////////////////////////////////////////////////
void DrawChar(int left, int top, char letter, unsigned short color)
{
int x, y;
int draw;
for(y = 0; y < 8; y++)
for (x = 0; x < 8; x++)
{
// grab a pixel from the font char
draw = font[(letter-32) * 64 + y * 8 + x];
// if pixel = 1, then draw it
if (draw)
DrawPixel3(left + x, top + y, color);
}
}
//////////////////////////////////////////////////
// Function: DrawPixel3
// Draws a pixel in mode 3
//////////////////////////////////////////////////
void DrawPixel3(int x, int y, unsigned short color)
{
videoBuffer[y * 240 + x] = color;

}
/////////////////////////////////////////////////////////////
// Function: DrawBox3
// Draws a filled box
/////////////////////////////////////////////////////////////
void DrawBox3(int left, int top, int right, int bottom,unsigned short color)
{
int x, y;
for(y = top; y < bottom; y++)
for(x = left; x < right; x++)
DrawPixel3(x, y, color);
}
/////////////////////////////////////////////////////////////
// Function: WaitVBlank
// Checks the scanline counter for the vertical blank period
/////////////////////////////////////////////////////////////
void WaitVBlank(void)
{
while(!(*ScanlineCounter));
while((*ScanlineCounter));
}
#endif
